<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2008 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_0/classes/prom_clan.php $
 * $Id: prom_clan.php 1360 2009-01-01 21:27:41Z Quietust $
 */

if (!defined('IN_GAME') && !defined('IN_TURNS'))
	die("Access denied");

require_once(PROM_BASEDIR .'classes/prom_entity.php');

define('RELATION_OUTBOUND', 1);
define('RELATION_INBOUND', 2);
define('RELATION_BOTH', RELATION_OUTBOUND | RELATION_INBOUND);

class prom_clan extends prom_entity
{
	// Constructor - initialize as Clan
	public function __construct($id = 0, $needlock = TRUE)
	{
		parent::__construct($id, $needlock, CLAN_TABLE, 'c_id', ENT_CLAN);
	}

	// Creates a brand new clan record and inserts it into the database
	// Must be called using an uninitialized clan object
	// Parameter $emp must be a locked empire record (for consistency, though it does not modify it)
	public function create ($emp, $name, $password)
	{
		if ($this->id != 0)
		{
			warning('Attempted to initialize already initialized entity of type '. get_class($this), 1);
			return FALSE;
		}

		if (get_class($emp) != 'prom_empire')
		{
			warning('Attempted to initialize entity of type '. get_class($this) .' using invalid base entity of type '. get_class($emp), 1);
			return FALSE;
		}

		if (!$emp->locked())
		{
			warning('Attempted to initialize entity of type '. get_class($this) .' using unlocked base entity of type '. get_class($emp), 1);
			return FALSE;
		}

		global $db;

		// creation of the clan and the lock should occur simultaneously to ensure the turns script doesn't steal the lock
		// this transaction will only work effectively if the clan table is declared as InnoDB; as MyISAM, this may fail
		$db->beginTransaction();
		$q = $db->prepare('INSERT INTO '. CLAN_TABLE.' (e_id_leader) VALUES (?)');
		db_bindIntValue($q, 1, $emp->e_id);
		if (!$q->execute())
		{
			warning('Failed to create clan for empire '. $emp, 1);
			return FALSE;
		}

		$this->id = $db->lastInsertId();
		if (!$this->lock(LOCK_NEW))
		{
			// ...so if the turns script does happen to be running, it may snatch up the lock before we were able to take it
			warning('Unable to lock newly created entity '. $this->id .' of type '. get_class($this), 1);
			// we can't allow this to fail, so mark the entity as locked anyways - the turn script should leave it alone
			$this->load();
			$this->lockid = LOCK_NEW;
		}
		$db->commit();

		// set various properties of the new clan
		$this->c_name = $name;
		$this->c_title = def_lang('CLAN_CREATE_DEFAULT_TITLE', $name);
		$this->c_password = md5($password);
		$this->c_members = 1;

		$q = $db->prepare('INSERT INTO '. CLAN_TOPIC_TABLE .' (c_id,ct_flags) VALUES (?,?)');
		db_bindIntValue($q, 1, $this->id);
		db_bindIntValue($q, 2, CTFLAG_NEWS);
		$q->execute() or warning('Failed to create news topic for clan '. $this, 1);
		$topic_id = $db->lastInsertId();

		$q = $db->prepare('INSERT INTO '. CLAN_MESSAGE_TABLE .' (ct_id,e_id,cm_body,cm_time) VALUES (?,?,?,?)');
		db_bindIntValue($q, 1, $topic_id);
		db_bindIntValue($q, 2, $emp->e_id);
		db_bindStrValue($q, 3, def_lang('CLAN_CREATE_DEFAULT_MOTD', $name));
		db_bindIntValue($q, 4, CUR_TIME);
		$q->execute() or warning('Failed to create news post for clan '. $this, 1);

		return TRUE;
	}

	// Retrieves a list of all relations this clan has with any others
	// checkflags - flags to examine (must include those in needflags)
	// needflags - exactly which flags need to be set or clear
	// direction - combination of RELATION_OUTBOUND and RELATION_INBOUND
	// full - TRUE to return the entire clan relation record, FALSE to simply return the other clan's ID
	public function listRelations ($checkflags, $needflags, $direction = RELATION_BOTH, $full = FALSE)
	{
		$relations = array();

		if ($this->id == 0)
		{
			warning('Attempted to list relations for uninitialized entity of type '. get_class($this), 1);
			return $relations;
		}

		global $db;

		if ($direction & RELATION_OUTBOUND)
		{
			$q = $db->prepare('SELECT * FROM '. CLAN_RELATION_TABLE .' WHERE c_id_1 = ? AND cr_flags & ? = ?');
			db_bindIntValue($q, 1, $this->id);
			db_bindIntValue($q, 2, $checkflags);
			db_bindIntValue($q, 3, $needflags);

			if (!$q->execute())
			{
				warning('Failed to fetch outward relations for clan '. $this, 1);
				return NULL;
			}
			while ($row = $q->fetch())
			{
				if ($full)
					$relations[$row['c_id_2']] = $row;
				else	$relations[] = $row['c_id_2'];
			}
		}

		if ($direction & RELATION_INBOUND)
		{
			$q = $db->prepare('SELECT * FROM '. CLAN_RELATION_TABLE .' WHERE c_id_2 = ? AND cr_flags & ? = ?');
			db_bindIntValue($q, 1, $this->id);
			db_bindIntValue($q, 2, $checkflags);
			db_bindIntValue($q, 3, $needflags);

			if (!$q->execute())
			{
				warning('Failed to fetch inward relations for clan '. $this, 1);
				return NULL;
			}
			while ($row = $q->fetch())
			{
				if ($full)
					$relations[$row['c_id_1']] = $row;
				else	$relations[] = $row['c_id_1'];
			}
		}

		return $relations;
	}

	// Shortcut function for listing all active alliances
	public function getAllies ()
	{
		if ($this->id == 0)
		{
			warning('Attempted to list allies for uninitialized entity of type '. get_class($this), 1);
			return array();
		}

		return $this->listRelations(CRFLAG_ALLY | CRFLAG_MUTUAL, CRFLAG_ALLY | CRFLAG_MUTUAL);
	}

	// Shortcut function for listing all active wars - ones that have been declared,
	// and ones that where peace has been requested (by the originator) but not yet accepted
	public function getWars ()
	{
		if ($this->id == 0)
		{
			warning('Attempted to list wars for uninitialized entity of type '. get_class($this), 1);
			return array();
		}

		$war1 = $this->listRelations(CRFLAG_WAR | CRFLAG_MUTUAL, CRFLAG_WAR | CRFLAG_MUTUAL);
		$war2 = $this->listRelations(CRFLAG_WAR | CRFLAG_MUTUAL, CRFLAG_WAR, RELATION_INBOUND);
		return array_merge($war1, $war2);
	}

	// Formats clan name+number as a string
	public function __toString ()
	{
		if ($this->id == 0)
			return lang('COMMON_CLAN_NAMEID', 'COMMON_CLAN_UNINITIALIZED', prenum(0));
		return lang('COMMON_CLAN_NAMEID', $this->c_name, prenum($this->c_id));
	}

	// Loads all clan names into an array
	public static function getNames ()
	{
		global $db;
		$q = $db->query('SELECT c_id, c_name FROM '. CLAN_TABLE .' ORDER BY c_id ASC') or warning('Failed to retrieve list of clan names', 1);
	
		$names = array();
		$names[0] = 'None';
		while ($c = $q->fetch())
			$names[$c['c_id']] = $c['c_name'];
	
		return $names;
	}

} // class prom_clan
?>
